//
//  UIColor+ImageGetColor.m
//  HyPopMenuView
//
//  Created by  H y on 15/9/8.
//  Copyright (c) 2015年 ouy.Aberi. All rights reserved.
//

#import "UIColor+ImageGetColor.h"

@implementation UIView (GetImgae)

- (UIImage*)imageRepresentation
{
    int width = CGRectGetWidth([self bounds]);
    int height = CGRectGetHeight([self bounds]);

    NSInteger myDataLength = width * height * 4;

    // allocate array and read pixels into it.
    GLubyte* buffer = (GLubyte*)malloc(myDataLength);
    glReadPixels(0, 0, width, height, GL_RGBA, GL_UNSIGNED_BYTE, buffer);

    // gl renders "upside down" so swap top to bottom into new array.
    // there's gotta be a better way, but this works.
    GLubyte* buffer2 = (GLubyte*)malloc(myDataLength);
    for (int y = 0; y < height; y++) {
        for (int x = 0; x < width * 4; x++) {
            buffer2[(height - 1 - y) * width * 4 + x] = buffer[y * 4 * width + x];
        }
    }

    // make data provider with data.
    CGDataProviderRef provider = CGDataProviderCreateWithData(NULL, buffer2, myDataLength, NULL);

    // prep the ingredients
    int bitsPerComponent = 8;
    int bitsPerPixel = 32;
    int bytesPerRow = 4 * width;
    CGColorSpaceRef colorSpaceRef = CGColorSpaceCreateDeviceRGB();
    CGBitmapInfo bitmapInfo = kCGBitmapByteOrderDefault;
    CGColorRenderingIntent renderingIntent = kCGRenderingIntentDefault;

    // make the cgimage
    CGImageRef imageRef = CGImageCreate(width, height, bitsPerComponent, bitsPerPixel, bytesPerRow, colorSpaceRef, bitmapInfo, provider, NULL, NO, renderingIntent);

    // then make the uiimage from that
    UIImage* myImage = [UIImage imageWithCGImage:imageRef];
    CGColorSpaceRelease(colorSpaceRef);
    CGDataProviderRelease(provider);
    CGImageRelease(imageRef);
    return myImage;
}

@end

@implementation UIColor (ImageGetColor)

+ (UIColor*)getPixelColorAtLocation:(CGPoint)point inImage:(UIImage*)image
{
    UIColor* color = nil;
    CGImageRef inImage = image.CGImage;
    CGContextRef cgctx = [self createARGBBitmapContextFromImage:
                                   inImage];

    if (cgctx == NULL) {
        return nil; /* error */
    }
    size_t w = CGImageGetWidth(inImage);
    size_t h = CGImageGetHeight(inImage);
    CGRect rect = { { 0, 0 }, { w, h } };

    CGContextDrawImage(cgctx, rect, inImage);

    unsigned char* data = CGBitmapContextGetData(cgctx);

    if (data != NULL) {
        int offset = 4 * ((w * round(point.y)) + round(point.x));
        int alpha = data[offset];
        int red = data[offset + 1];
        int green = data[offset + 2];
        int blue = data[offset + 3];
//        NSLog(@"offset: %i colors: RGB A %i %i %i  %i", offset, red, green,
//            blue, alpha);

//        NSLog(@"x:%f y:%f", point.x, point.y);

        color = [UIColor colorWithRed:(red / 255.0f) green:(green / 255.0f) blue:
                                                                                (blue / 255.0f)
                                alpha:(alpha / 255.0f)];
    }

    CGContextRelease(cgctx);

    if (data) {
        free(data);
    }

    return color;
}

+ (CGContextRef)createARGBBitmapContextFromImage:(CGImageRef)inImage
{

    CGContextRef context = NULL;

    CGColorSpaceRef colorSpace;

    void* bitmapData;

    unsigned long bitmapByteCount;

    unsigned long bitmapBytesPerRow;

    size_t pixelsWide = CGImageGetWidth(inImage);

    size_t pixelsHigh = CGImageGetHeight(inImage);

    bitmapBytesPerRow = (pixelsWide * 4);

    bitmapByteCount = (bitmapBytesPerRow * pixelsHigh);

    colorSpace = CGColorSpaceCreateDeviceRGB();

    if (colorSpace == NULL)

    {

        fprintf(stderr, "Error allocating color space\n");

        return NULL;
    }

    bitmapData = malloc(bitmapByteCount);

    if (bitmapData == NULL)

    {

        fprintf(stderr, "Memory not allocated!");

        CGColorSpaceRelease(colorSpace);

        return NULL;
    }

    context = CGBitmapContextCreate(bitmapData,

        pixelsWide,

        pixelsHigh,

        8,

        bitmapBytesPerRow,

        colorSpace,

        (CGBitmapInfo)kCGImageAlphaPremultipliedFirst);

    if (context == NULL)

    {

        free(bitmapData);

        fprintf(stderr, "Context not created!");
    }

    CGColorSpaceRelease(colorSpace);

    return context;
}

@end

@implementation UIImage (Tint)
- (UIImage*)imageWithTintColor:(UIColor*)tintColor
{
    //We want to keep alpha, set opaque to NO; Use 0.0f for scale to use the scale factor of the device’s main screen.
    UIGraphicsBeginImageContextWithOptions(self.size, NO, 0.0f);
    [tintColor setFill];
    CGRect bounds = CGRectMake(0, 0, self.size.width, self.size.height);
    UIRectFill(bounds);

    //Draw the tinted image in context
    [self drawInRect:bounds blendMode:kCGBlendModeDestinationIn alpha:1.0f];

    UIImage* tintedImage = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();

    return tintedImage;
}
@end
